﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ConsoleApplication1
{
    public class FarmRow : IFarmRow
    {
        private IList<ICell> cells = new List<ICell>();
        private string rowString = string.Empty;

        public FarmRow(string rowString, IFarm relatedFarm)
        {
            this.RelatedFarm = relatedFarm;
            this.RelatedFarm.AddRow(this);
            this.RowString = rowString;
        }

        #region IFarmRow Members

        public int ColumnCount
        {
            get
            {
                return this.RowString.Length;
            }
        }

        public IFarm RelatedFarm
        {
            get;
            set;
        }

        public string RowString
        {
            get
            {
                return rowString;
            }
            set
            {
                rowString = value;
                for (int i = 0; i < rowString.Length; i++)
                {
                    this.AddCell(new Cell(int.Parse(rowString[i].ToString())));
                }
            }
        }

        public ICell this[int yIndex]
        {
            get
            {
                if (yIndex < cells.Count)
                {
                    return cells[yIndex];
                }
                else
                {
                    throw new IndexOutOfRangeException();
                }
            }
            set
            {
                if (yIndex < cells.Count)
                {
                    cells[yIndex] = value;
                }
                else
                {
                    throw new IndexOutOfRangeException();
                }
            }
        }

        void AddCell(ICell cell)
        {
            cells.Add(cell);
            int i = this.RelatedFarm.RowCount - 1;
            int j = this.cells.Count - 1;

            if (i == 0 && j > 0)
            {
                this.RelatedFarm[0][j - 1].RelatedCells.Add(cell);
                cell.RelatedCells.Add(this.RelatedFarm[0][j - 1]);
            }
            if (i > 0)
            {
                if (j > 0)
                {
                    this.RelatedFarm[i - 1][j - 1].RelatedCells.Add(cell);
                    cell.RelatedCells.Add(this.RelatedFarm[i - 1][j - 1]);
                    this.RelatedFarm[i][j - 1].RelatedCells.Add(cell);
                    cell.RelatedCells.Add(this.RelatedFarm[i][j - 1]);
                }
                this.RelatedFarm[i - 1][j].RelatedCells.Add(cell);
                cell.RelatedCells.Add(this.RelatedFarm[i - 1][j]);
                if (j + 1 < RowString.Length)
                {
                    this.RelatedFarm[i - 1][j + 1].RelatedCells.Add(cell);
                    cell.RelatedCells.Add(this.RelatedFarm[i - 1][j + 1]);
                }

            }
            foreach (ICell _cell in cells)
            {
                _cell.SpecialNumberUpdate();
            }
        }

        #endregion

        #region IEnumerable<ICell> Members

        public IEnumerator<ICell> GetEnumerator()
        {
            return new FarmRowEnum(this.cells);
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return new FarmRowEnum(this.cells);
        }

        #endregion
    }
}
